[TOC]
# Introduction

We all remember problems we had in school, stated likes:

> Alice recieve 10 candies from her grandmother and later gives 2 candies to Bob.
> How many candies has Alice left ?

We will walk through ERP5 business model and Inventory API using that problem as an example.
The 5 base classes of ERP5, also called _Universal Business Model_ are shown in the picture below.

![ERP5's 5 classes](https://www.erp5.com/developer-ERP5.UBM.Picture?quality=75.0&display=small&format=png "ERP5 5 classes")

Using ERP5 Universal Business Model this simple problem will be modeled using:
 - *resource:* The resource are candies
 - *node:* Alice, Bob and the Grandmother are nodes
 - *movements:* When Alice gives 2 candies to Bob, there is a movement where Alice is _source_ and Bob is _destination_. The resource of this movement is candy.

In our problem, there are two movements:

| Movement | Source        | Destination   | Resource | Quantity  |
| -------- | ------------- |-------------- | -------- | ---------:|
|       M1 | Grandmother   | Alice         | Candy    |        10 |
|       M2 | Alice         | Bob           | Candy    |         2 |


A simple solution to "how many candies Alice has left" is achived by summing the quantities of incoming movements, ie. movements for which Alice is the destination, and then substracting the quantities of outgoing movements, ie. movements for which Alice is the source.

Which in pseudo SQL format, means:

```sql
SELECT SUM(quantity) FROM movements WHERE Source = 'Alice'
- SUM(quantity) FROM movements Where Destination = 'Alice'
```

Which leads to complex and unefficient queries.

The underlying idea of Inventory is to see all movements from the source's point of view and from the destination's point of view. We can say that an incoming movement increase the inventory of the destination *node* while decreasing the inventory of the node at the other side of the movement, that we call *mirror node* in Inventory API terminology.

We store the movements using the following data structure:

| Movement | Node          | Mirror Node   | Resource | Quantity  |
| -------- | ------------- | ------------- | -------- | ---------:|
|       M1 | Grandmother   | Alice         | Candy    |       -10 |
|       M1 | Alice         | Grandmother   | Candy    |        10 |
|       M2 | Alice         | Bob           | Candy    |        -2 |
|       M2 | Bob           | Alice         | Candy    |         2 |


Using this data structure querying inventory of Alice is much efficient:

```sql
SELECT SUM(quantity) FROM stock WHERE Node = 'Alice'
```

Using ERP5 API, this call would be :

```python
return portal_simulation.getInventory(
   node_uid=alice.getUid()
)
```

# Dates

We are usually interested in knowing the stock level at a certain point of time. This is rather easy, because we usually record the point in time when the movement occured.

It may happen that the movement itself takes some time and that there is a difference between the time when resource beeing leaves the source and the time when it arrives at destination. 

To support this, movements have two properties:

 - *Start Date*: When the movement begins and the quantity is removed from source
 - *Stop Date*: When the movements finishes and the quantity is added at destination


Let's imagine that on Wednesday Alice send 3 candies to her cousin Carol that lives far away. Carol will only recieve the candies after two days, because it is the time the post office will take to deliver. This movement is depicted as M3 in the table below:


| Movement | Node          | Mirror Node   | Resource | Quantity  | Date      |
| -------- | ------------- | ------------- | -------- | ---------:| --------- |
|       M1 | Grandmother   | Alice         | Candy    |       -10 | Monday    |
|       M1 | Alice         | Grandmother   | Candy    |        10 | Monday    |
|       M2 | Alice         | Bob           | Candy    |        -2 | Tuesday   |
|       M2 | Bob           | Alice         | Candy    |         2 | Tuesday   |
|       M3 | Alice         | Carol         | Candy    |        -3 | Wednesday |
|       M3 | Carol         | Alice         | Candy    |         3 | Thursday  |

In this table it, as easy to see that before Tuesday evening, Alice had 8 candies:

```python
>>> getInventory(node=Alice, at_date=Tuesday)
8 # M1 & M2
```

Because under the hood, this will use a query like:
```sql
SELECT SUM(quantity) from stock where Node='Alice' and Date <= Monday
```

On Wednesday evening, has only 5 Candies because the movement already decreased her stock

```python
>>> getInventory(node=Alice, at_date=Tuesday)
5 # M1 & M2 & M3
```
```sql
SELECT SUM(quantity) from stock where Node='Alice' and Date <= Tuesday
```

But Carol's inventory of candies is still 0 on Wednesday:
```python
>>> getInventory(node=Carol, at_date=Wednesday)
0
```
```sql
SELECT SUM(quantity) from stock where Node='Carol' and Date <= Tuesday
```

And this inventory is only increased on Thursday:
```python
>>> getInventory(node=Carol, at_date=Thursday)
3
```
```sql
SELECT SUM(quantity) from stock where Node='Carol' and Date <= Thursday
```

---

# Complete structure of a movement

## Movements properties
In our candy example, we abstracted many of properties that are needed on movements to represent the complexity of real business cases.

A more realistic example could be ``20 kilograms of *sand* delivered from supplier to warehouse where the unit price is 10€``.

In that example movement, supplier organisation is the *source*, warehouse is the *destination*. The *resource* is sand,  *quantity* is 20, the *quantity unit* is Kilogram, the *price* (per unit) is 10 and the *price currency* is the Euro currency. Quantity and Price properties are set on movements and we can calculate the *total price* by multiplying the quantity by the price. In that case, total price is $20*10 = 200$.

There are movements of different types that are stored in different modules, for example there are order, delivery or invoices movements. We also separate sales, purchase and internal movements. All movements have a *portal_type* property that can be for example `Purchase Order Line` or `Sale Order Line`. Portal Type comes from Zope CMF and is used to configure technical aspects such as the available view and the applicable security rules, so there is an higher level to define what is the kind of movement; this is the *use* category.

Movements have a *simulation_state* which is controlled by a simulation workflow. Movements are usually part of a *delivery* on which users with the proper accreditation can pass workflow transitions to change the state. For example, one sales agent can "confirm" a sales order, that will change the simulation state of all the movements from planned to confirmed. Typically, simulation states are used for different level of certainty and have different level of planning.

Movements have a *start_date* and a *stop_date* properties whose types are Zope DateTime instances.

Even if in our example the total price is 200€, the "value" of this movement can be different for the supplier and for the warehouse. The supplier may want to account that the value of these 10Kg of Sand was, let's say, 110€. The *source_total_asset_price* property of a movement holds the value of this movement in the source's inventory and symetrically, the *destination_total_asset_price* the value in destination's inventory. Source asset price is always in the source's accounting currency and destination asset price always in destination's accounting currency.

Movements also have an *efficiency* property to note that a percentage of the movements quantity can be lost during the movement. For example, a movement with quantity 10 and efficiency of 95% would decrease the source's inventory by 10 and increase the inventory of the destination by 9.5. This is what we call source *inventoriated quantity* and *destination inventoriated quantity*.

## Difference from Amount and Movement

Amount is used to represent a quantity of a given resource in a given quantity unit. Amounts are used in Movements to define a subtraction of a resource from a node and its addition to another node (mirror_node). 

For more information check the [Movement interface documentation](https://lab.nexedi.com/nexedi/erp5/blob/master/product/ERP5/interfaces/movement.py).

## Movements categories

Previous example just described a movement going from supplier to warehouse. The *physical* location of the resource changed from supplier to the warehouse, but we want to consider other parties. For example, the *ownership* might not be to the warehouse but be the headquarters organisation. We can also states things like "Mr Smith is the main contact for questions related to this sales"; this is *administration*; or "the payment will be made on this bank account"; this is *payment*; or "this sand will be used for the project of XXX (better resource)"; this is *project* or it that "it will be used for production operations"; this is *function*; that it will be paid using a governement grant money; this is *funding*.

Movements have various source / destination categories and it seems that this list of categories depends on the kind of business being modeled. Some examples are summarized in the table below:

| Category           | Meaning |
|--------------------|---------|
|Source/Destination  | What is the physical location |
|Source Section/Destination Section | Who has ownership after this movement |
|Source Payment/Destination Payment | Which "bank account" is used to pay / who will pay |
|Source Project/Destination Project | What is the project for the source or destination |
|Source Function/Destination Function | What is the "purpose" for the source or destination |
|Source Administration/Destination Administration | Who is in charge of administrative tasks related to this movement |


Physical location is queried using *node* inventory API parameter.
To query ownership, we use *section*.
To query payment, we use *payment*.
For the project, we use *project*.



XXX vector projection


# Different kind of inventories
Depending of the level of certainty, we usually consider different ways of seeing the inventory.

## Current Inventory
Current inventory is what is currently in the stock. This includes all movements that have a simulation state representing the fact that this movement has already happened.
To calculate current inventory, we take into account *transit* movements with special care. A movement that is in *started* state has already decreased the inventory of the source, but not yet increased the inventory of the destination.
Current inventory is for example used for the actual quantities of goods in the warehouses.

## Future Inventory
Future inventory considers all movements that have not really happened yet, but for which we already know that their are great changes that they happen. Future inventory takes into account *planned* or *confirmed* state and is generally used to make stocks previsions.

## Available Inventory
Available inventory is the inventory that can be offered to customers without risking to go out of stock. Available inventory counts incoming movements only if they really happened and count outgoing movements even if they are only planned.

## Relationship with budget

Budgets are documents that implement a process of forecasting and later comparing the real values with what was planned. The budget is used to define a limit total quantity or price of a set of movements between nodes. When movements are created they can be matched to the related budgets by using predicates. This can be used to check budget consumption and use it as a constraint to prevent the execution of movements that would exceed the budget.

# Hierarchical categories
Explain node_category instead of node_uid


# Complete API
XXX explain that the idea is to pass parameter to represent a "perimeter" to consider.
## getInventory
`getInventory` sums the quantity of all movements to return the quantity of resources.
XXX more ?
This method does not check that the provided parameters leads to results that make sense in the real world. With this method, one can request the sum of kilograms of carots + litters of gasoline.
## getInventoryAssetPrice
`getInventoryAssetPrice` returns the total "values" XXX of the movements. 

## getInventoryList
`getInventoryList` can query in only one call the inventories of multiple nodes, it is equivalent to the *GROUP BY* SQL Statement.

The aggregation parameters has to be specified. For example, in the case of warehouse inventories, we want to pass `group_by_node=True` and `group_by_resource=True` to get the inventory for each node and resource.


Using one `getInventoryList` is usually much more efficient than doing multiple `getInventory`/`getInventoryAssetPrice` calls. 
For example, this code:
```python
print "Alice's has {inventory} candies, value is {value}".format(
        inventory=getInventory(node_uid=alice.getUid(), resource=candy.getUid(),
        asset_price=getInventoryAssetPrice(node_uid=alice.getUid(), resource_uid=candy.getUid()))
print "Bob's has {inventory} candies, value is {value}".format(
        inventory=getInventory(node_uid=bob.getUid(), resource=candy.getUid(),
        asset_price=getInventoryAssetPrice(node_uid=bob.getUid(), resource_uid=candy.getUid()))
```

can be written more efficiently with `getInventoryList` this way:

```python
for brain in getInventoryList(node_uid=(alice.getUid(), bob.getUid()),
                              resource_uid=candy.getUid(),
                              group_by_node=True):
    print "{node}'s inventory of candy is {inventory}, value is {value}".format(
        node=brain.node_title,
        inventory=brain.total_quantity,
        asset_price=brain.total_price)
```

As we saw in the above example, the returned values of getInventoryList are a sequence of `InventoryListBrain` objects.
These `InventoryListBrain` have the following attributes:

| Attribute      | Description |
|----------------|-------------|
| total_quantity | The sum of quantities of all movements from this group. |
| total_price    | The sum of asset price of all movements from this group. |
| node_uid       | The uid of the node. Only makes sense if using `group_by_node`. Otherwise the `node_uid` of one random movement from the group will be returned ( because the underlying query relies on MySQL not using [`ONLY_FULL_GROUP_BY`](https://dev.mysql.com/doc/refman/5.7/en/group-by-handling.html) here ) |
| node_title     | The title of the node. Only makes sense if using `group_by_node`. |
| resource_uid   | The uid of the resource. Only makes sense if using `group_by_resource`. |
| resource_title | The title of the resource. Only makes sense if using `group_by_resource`. |
| section_uid    | The uid of the section. Only makes sense if using `group_by_section`. |
| section_title  | The title of the section. Only makes sense if using `group_by_section`. |
| project_uid    | The uid of the project. Only makes sense if using `group_by_project`. |
| mirror_section_uid | The uid of the mirror section. Only makes sense if using `group_by_mirror_section`. |
| mirror_node_uid | The uid of the mirror node. Only makes sense if using `group_by_mirror_node`. |
| uid            | The uid of one random movement from the group. |
| path            | The path of one random movement from the group. |
| getObject()    | Returns a random movement from the group. |
| parameters from `select_list` | All parameters that have been passed to `select_list` can also be selected. |
| everything else | An implicit `getObject` will be called and the attribute will be fetched from a movement. This can lead to errors. For example, do not use `brain.getQuantity()` or `brain.quantity` which will just return quantity of a random movement regardess of aggregation |


## getMovementHistoryList
`getMovementHistoryList` list all movements that form the inventory. The returned value is a list of `MovementHistoryListBrain` instances.

Continuing our example, `getMovementHistoryList` can be used to retrieve all the movements corresponding to an inventory, this can be used to explain
```python
print "Alice's has {inventory} candies".format(
        inventory=getInventory(node_uid=alice.getUid(), resource_uid=candy.getUid())

print "The history is:"
for brain in getMovementHistoryList(node_uid=alice.getUid(), resource_uid=candy.getUid()):
  if brain.quantity > 0:
    print "On {date}, she recieved {quantity} candies from {mirror_section_title}".format(
      date=brain.date,
      quantity=brain.quantity,
      mirror_section_title=brain.mirror_section_title)
  else:
    print "On {date}, she gave {quantity} candies to {mirror_section_title}".format(
      date=brain.date,
      quantity=brain.quantity,
      mirror_section_title=brain.mirror_section_title)
```

## Complete List of supported parameters

### date
- from_date: only take into account rows with date >= *from_date*
- to_date: only take into account rows with date < *to_date*
- at_date: only take into account rows with date <= *at_date*
- mirror_date: XXX

### perimeter definition

XXX check https://www.erp5.com/erp5-TechnicalNote.Omit.Input.Omit.Output.In.Inventory.API


#### specifing node
- node: only take into account rows which ode_uid* is equivalent to *node*
- payment: only take into account rows which *payment_uid* is equivalent to *payment*
- section: only take into account rows which *section_uid* is equivalent to *section*
- mirror_section: only take into account rows which *mirror_section_uid* is equivalent to *mirror_section*

#### specifying node category
- resource_category: only take into account rows where *resource_uid* is member of *resource_category*
- node_category: only take into account rows which *node_uid* is member of *section_category*
- payment_category: only take into account rows which *payment_uid* is a member of *section_category*
- section_category: only take into account rows which *section_uid* is member of *section_category*
- mirror_section_category: only take into account rows which *mirror_section_uid* is member of *mirror_section_category*

XXX note on performance, instead of a for example a node_category query, if the number of nodes is small enough, it is usually better to do a first catalog search to find all the nodes matching node_category and pass them as node_uid. The same applies for all `_category` parameters, node was an example here.

#### filter
( XXX is this really implented ?)
- node_filter: only take into account rows which *node_uid* matches *node_filter*
- payment_filter: only take into account rows which *payment_uid* matches *payment_filter*
- section_filter: only take into account rows which *section_uid* matches *section_filter*
- mirror_section_filter: only take into account rows which *mirror_section_uid* matches *mirror_section_filter*

#### omit
Allows to only consider incoming or outgoing movements.
- omit_input: do not take into account movement with *quantity > 0*
- omit_output: do not take into account movement with *quantity < 0*
- omit_asset_increase: do not take into account movement with *asset_price > 0*
- omit_asset_decrease: do not take into account movement with *asset_price < 0*

#### variation
- variation_text:  only take into account rows with specified *variation_text*. It is implemented in a very far from perfect way and had poor performance. **(XXX: what is this?)**
- sub_variation_text: ???
- variation_category: only take into account rows with matching *variation* or where its *variation* is a member of this list
- ignore_variation: do not take into account variation in inventory calculation **(XXX: what is this?)**

#### movement states
- simulation_state: only take into account rows with the specified *simulation_state*
- transit_simulation_state: only take into account rows which states are transit states 
- omit_transit: do not evaluate *transit_simulation_state*
- input_simulation_state: only take into account rows with specified *simulation_state* and *quantity > 0* **(XXX: why do we have duplicated parameters? this can be achieved by using two existing parameters, for i.e., this can be reproduced by using *omit_output* and *simulation_state*)**
- output_simulation_state: only take into account rows with specified *simulation_state* and *quantity < 0* **(XXX: same as line above)**
- omit_simulation: do not take into account simulation movements

#### grouping
- group_by_variation: group results by variataion
- group_by_node: group results by node
- group_by_mirror: group results by mirror
- group_by_sub_variation: group results by sub variation
- group_by_movement: group results by movement
- group_by: a list of columns supported by catalog API.

Grouping parameters are for getInventoryList, but they also can be applied to getMovementHistoryList.


#### misc
- only_accountable: only take into account accountable movements. Default is *True*
- precision: the precision used to round quantities and prices
- metric_type: convert results to an specific *metric_type*
- quantity_unit: display results using this specific unit
- transformed_resource: the resource or list of resource that can be produced using the resulting movement's resource as an input. *relative_resource_url* for each line will point to the transformed resource, while the stock will be expressed in number of the respective transformed resources **(XXX: what does this actually mean?)**
- transformed_variation_text: to be used with *transformed_resource* to refine the transformation selection only to those using variated resources as input

#### Queries

Note that all inventory API methods supports `Query` arguments, which allows to do advanced queries.

For example, to get movements going to A or to nowhere, you can do:

```python
from Products.ZSQLCatalog.SQLCatalog import SimpleQuery, ComplexQuery

portal.portal_simulation.getMovementHistoryList(
  query=ComplexQuery([
      SimpleQuery(node_uid=A.getUid()),
      SimpleQuery(node_uid=None)],
    operator="OR")) 
```

TODO: this should be covered in testInventoryAPI as well.


# Inventory "documents"

Whereas we calculate inventory by summing all the movements to date, there may be differences between the theorical stock in the database and the real stock on the warehouse floor. It is common business practice to make so called "inventory" (XXX check in dictionary) where people go on the floor and count how many items of each resource are physically present in each stock locations.
After this, they can input the actual values in an `Inventory` document.

Let's consider an example.
Many incoming purchase packing lists have increased the stocks and many outgoing sales packing lists have decreased the stock. The stock levels are as displayed in this table:

| Node          | Resource | Quantity  |
| ------------- | -------- | ---------:|
| N1            | R1       |         5 |
| N1            | R2       |        10 |
| N2            | R1       |         3 |

When counting the quantities of R1 and R2 in nodes N1 and N2, we notice that the physical stock is different from the sum of movements, probably because some products were lost / stolen or there have been some mistakes.

The physical stock is:

| Node          | Resource | Quantity  |
| ------------- | -------- | ---------:|
| N1            | R1       |         4 |
| N1            | R2       |        12 |
| N2            | R1       |         3 |

This is the information that will be entered in an inventory document and this inventory will be "indexed" in the stock table in a way that it adjust the quantities in the stock table to match the physical stock, something like this:

| Node          | Resource | Quantity  |
| ------------- | -------- | ---------:|
| N1            | R1       |        -1 |
| N1            | R2       |         3 |

Then, when we call `getInventory(node=N1, resource=R1)`, the following lines are considered:

| Node          | Resource | Quantity  |
| ------------- | -------- | ---------:|
| *N1*          | *R1*     |       *5* |
| N1            | R2       |        10 |
| N2            | R1       |         3 |
| *N1*          | *R1*     |      *-1* |
| N1            | R2       |         3 |

And the value returned by `getInventory` is 4, which matches what has been counted on the floor during inventory procedure.


XXX what about price for Inventory ? Is inventory a movement or a path ?


# Flow API
TODO: interpolation API


# Value of stocks
TODO: explain purchase price, evaluation methods (FIFO, weighted average, etc)


# Example use cases

## Warehouse Inventory

## Time 



# Tracking API

XXX or another doc ?
